#ifndef __CGAME__
#define __CGAME__

#include "d3d12.h"
#include "dxgi.h"
#include "dxgi1_4.h"
#include "d3dx12.h"
#include "UploadBuffer.h"
#include "DirectXMath.h"
#include "d3dcompiler.h"
#include "external\WRL\ComPTR.h"

using namespace DirectX;

struct ConstantBufferData
{
	XMMATRIX matrixWorld;
	XMMATRIX matrixView;
	XMMATRIX matrixProjection;
};

class CGame
{
	static const int SwapChainCount = 2;

	ComPtr<ID3D12Debug> m_D3D12DebugController;
	ComPtr<IDXGIFactory4> m_DXGIFactory;
	ComPtr<ID3D12Device> m_Dev;
	ComPtr<IDXGISwapChain> m_SwapChain;
	ComPtr<ID3D12Fence> m_Fence;

	ComPtr<ID3D12CommandQueue> m_CmdQueue;
	ComPtr<ID3D12CommandAllocator> m_CmdAllocator;
	ComPtr<ID3D12GraphicsCommandList> m_GfxCmdList;

    ComPtr<ID3D12CommandAllocator> m_CmdAllocatorDraw;
    ComPtr<ID3D12GraphicsCommandList> m_DrawCmdList;

	ComPtr<ID3D12DescriptorHeap> m_RtvHeap;
	ComPtr<ID3D12DescriptorHeap> m_DSHeap;
	ComPtr<ID3D12DescriptorHeap> m_CBVHeap;

	ComPtr<ID3D12Resource> m_SwapChainBuffer[SwapChainCount];
	ComPtr<ID3D12Resource> m_DepthStencilBuffer;

	ComPtr<ID3D12Resource> m_VertexBufferGPU = NULL;
	ComPtr<ID3D12Resource> m_VertexBufferUploader = NULL;

	ComPtr<ID3D12Resource> m_IndexBufferGPU = NULL;
	ComPtr<ID3D12Resource> m_IndexBufferUploader = NULL;

	UploadBuffer<ConstantBufferData>* m_ObjectCB;

	ComPtr<ID3D12RootSignature> m_RootSignature;

	ComPtr<ID3DBlob> m_VSByteCode;
	ComPtr<ID3DBlob> m_PSByteCode;
	D3D12_INPUT_ELEMENT_DESC m_InputLayout[2];

	ComPtr<ID3D12PipelineState> m_PipelineStateObject;

public:

	void Initialize(void * windowHandle, int width, int height);
	void Update();
	void Render();
    void Shutdown();

private:

	void EnableD3D12DebugMode();
	void CreateCommandObjects();
	void CreateSwapChain();
	void CreateRtvAndDSHeaps();
	void CreateBackbufferRtvAndDsv();

	void SetupViewportAndScissorRects();

	void FlushCommandQueue();

	void CreateConstantBuffer();
	void CreateRootSignature();
	void CreateShadersAndInputLayout();
	void CreateVertexBuffer();
	void CreateIndexBuffer();
	void CreatePSO();
    void CreateDrawCommandList();

	ID3D12Resource* CurrentBackBuffer() const;
	D3D12_CPU_DESCRIPTOR_HANDLE CurrentBackBufferView() const;
	D3D12_CPU_DESCRIPTOR_HANDLE DepthStencilView() const;

	void* m_WindowHandle;
	int m_Width;
	int m_Height;
	float m_AspectRatio;
	int m_CurrBackBuffer = 0;
	int m_RtvDescriptorSize;
	int m_DsvDescriptorSize;
	int m_CurrentFence;
};

#endif